@@ -19,7 +19,7 @@
  * Boston, MA 02110-1301 USA
  *
  * Authors:
- * 	Arjan van de Ven <arjan@linux.intel.com>
+ *	Arjan van de Ven <arjan@linux.intel.com>
  */
 
 #include <getopt.h>
@@ -35,11 +35,11 @@
 #include <assert.h>
 #include <locale.h>
 #include <time.h>
+#include <limits.h>
 #include <sys/stat.h>
 
 #include "powertop.h"
 
-#define VERSION "1.11"
 
 uint64_t start_usage[8], start_duration[8];
 uint64_t last_usage[8], last_duration[8];
@@ -62,7 +62,6 @@ struct irqdata {
 	int active;
 	int number;
 	uint64_t count;
-	char description[256];
 };
 
 struct irqdata interrupts[IRQCOUNT];
@@ -98,27 +97,33 @@ void push_line(char *string, int count)
 		}
 	if (linehead == linesize)
 		lines = realloc (lines, (linesize ? (linesize *= 2) : (linesize = 64)) * sizeof (struct line));
+	memset(&lines[linehead], 0, sizeof(&lines[0]));
 	lines[linehead].string = strdup (string);
 	lines[linehead].count = count;
+	lines[linehead].disk_count = 0;
 	lines[linehead].pid[0] = 0;
 	linehead++;
 }
 
-void push_line_pid(char *string, int count, char *pid) 
+void push_line_pid(char *string, int cpu_count, int disk_count, char *pid)
 {
 	int i;
 	assert(string != NULL);
+	assert(strlen(string) > 0);
 	for (i = 0; i < linehead; i++)
 		if (strcmp(string, lines[i].string) == 0) {
-			lines[i].count += count;
+			lines[i].count += cpu_count;
+			lines[i].disk_count += disk_count;
 			if (pid && strcmp(lines[i].pid, pid)!=0)
 				lines[i].pid[0] = 0;
 			return;
 		}
 	if (linehead == linesize)
 		lines = realloc (lines, (linesize ? (linesize *= 2) : (linesize = 64)) * sizeof (struct line));
+	memset(&lines[linehead], 0, sizeof(&lines[0]));
 	lines[linehead].string = strdup (string);
-	lines[linehead].count = count;
+	lines[linehead].count = cpu_count;
+	lines[linehead].disk_count = disk_count;
 	if (pid)
 		strcpy(lines[linehead].pid, pid);
 	linehead++;
@@ -165,12 +170,61 @@ int update_irq(int irq, uint64_t count, char *name)
 	interrupts[firstfree].active = 1;
 	interrupts[firstfree].count = count;
 	interrupts[firstfree].number = irq;
-	strcpy(interrupts[firstfree].description, name);
-	if (strcmp(name,"i8042\n")==0)
-		strcpy(interrupts[firstfree].description, _("PS/2 keyboard/mouse/touchpad"));
 	return count;
 }
 
+static int percpu_hpet_timer(char *name)
+{
+	static int timer_list_read;
+	static int percpu_hpet_start = INT_MAX, percpu_hpet_end = INT_MIN;
+	char *c;
+	long hpet_chan;
+
+	if (!timer_list_read) {
+		char file_name[20];
+		char ln[80];
+		FILE *fp;
+
+		timer_list_read = 1;
+		snprintf(file_name, sizeof(file_name), "/proc/timer_list");
+		fp = fopen(file_name, "r");
+		if (fp == NULL)
+			return 0;
+
+		while (fgets(ln, sizeof(ln), fp) != NULL)
+		{
+			c = strstr(ln, "Clock Event Device: hpet");
+			if (!c)
+				continue;
+
+			c += 24;
+			if (!isdigit(c[0]))
+				continue;
+
+			hpet_chan = strtol(c, NULL, 10);
+			if (hpet_chan < percpu_hpet_start)
+				percpu_hpet_start = hpet_chan;
+			if (hpet_chan > percpu_hpet_end)
+				percpu_hpet_end = hpet_chan;
+		}
+		fclose(fp);
+	}
+
+	c = strstr(name, "hpet");
+	if (!c)
+		return 0;
+
+	c += 4;
+	if (!isdigit(c[0]))
+		return 0;
+
+	hpet_chan = strtol(c, NULL, 10);
+	if (percpu_hpet_start <= hpet_chan && hpet_chan <= percpu_hpet_end)
+		return 1;
+
+	return 0;
+}
+
 static void do_proc_irq(void)
 {
 	FILE *file;
@@ -178,7 +232,7 @@ static void do_proc_irq(void)
 	char line2[1024];
 	char *name;
 	uint64_t delta;
-	
+
 	interrupt_0 = 0;
 	total_interrupt  = 0;
 
@@ -191,13 +245,14 @@ static void do_proc_irq(void)
 		uint64_t count = 0;
 		int special = 0;
 		memset(line, 0, sizeof(line));
+		memset(line2, 0, sizeof(line));
 		if (fgets(line, 1024, file) == NULL)
 			break;
 		c = strchr(line, ':');
 		if (!c)
 			continue;
 		/* deal with NMI and the like.. make up fake nrs */
-		if (line[0] != ' ' && (line[0] < '0' || line[0] > '9')) {	
+		if (line[0] != ' ' && (line[0] < '0' || line[0] > '9')) {
 			if (strncmp(line,"NMI:", 4)==0)
 				nr=20000;
 			if (strncmp(line,"RES:", 4)==0)
@@ -228,13 +283,13 @@ static void do_proc_irq(void)
 			c = newc;
 		}
 		c = strchr(c, ' ');
-		if (!c) 
+		if (!c)
 			continue;
 		while (c && *c == ' ')
 			c++;
 		if (!special) {
 			c = strchr(c, ' ');
-			if (!c) 
+			if (!c)
 				continue;
 			while (c && *c == ' ')
 				c++;
@@ -244,14 +299,26 @@ static void do_proc_irq(void)
 		c = strchr(name, '\n');
 		if (c)
 			*c = 0;
-		if (strcmp(name, "i8042")) { 
-			if (special) 
-				sprintf(line2, _("   <kernel IPI> : %s"), name);
+
+		/* deal with multi-queue NICs */
+		if (strncmp(name, "eth",3) == 0 && strchr(name,' ') == NULL) {
+			c = strchr(name, '-');
+			if (c)
+				*c = 0;
+		}
+
+		if (strcmp(name, "i8042")) {
+			if (special)
+				sprintf(line2, _("[%s] <kernel IPI>"), name);
 			else
-				sprintf(line2, _("    <interrupt> : %s"), name);
+				sprintf(line2, _("[%s] <interrupt>"), name);
 		}
 		else
-			sprintf(line2, _("    <interrupt> : %s"), _("PS/2 keyboard/mouse/touchpad"));
+			sprintf(line2, _("%s interrupt"), _("PS/2 keyboard/mouse/touchpad"));
+
+		/* skip per CPU timer interrupts */
+		if (percpu_hpet_timer(name))
+			delta = 0;
 
 		if (nr > 0 && delta > 0)
 			push_line(line2, delta);
@@ -362,7 +429,7 @@ static void read_data_cpuidle(uint64_t * usage, uint64_t * duration)
 				if (f == NULL)
 					break;
 
-			
+
 				f = strstr(line, "MWAIT ");
 				if (f) {
 					f += 6;
@@ -400,7 +467,7 @@ static void read_data_cpuidle(uint64_t * usage, uint64_t * duration)
 			file = fopen(filename, "r");
 			if (!file)
 				continue;
-		
+
 			memset(line, 0, 4096);
 			f = fgets(line, 4096, file);
 			fclose(file);
@@ -412,7 +479,7 @@ static void read_data_cpuidle(uint64_t * usage, uint64_t * duration)
 			clevel++;
 			if (clevel > maxcstate)
 				maxcstate = clevel;
-		
+
 		}
 		closedir(dir);
 
@@ -429,7 +496,7 @@ static void read_data(uint64_t * usage, uint64_t * duration)
 	r = stat("/sys/devices/system/cpu/cpu0/cpuidle", &s);
 	if (!r) {
 		read_data_cpuidle(usage, duration);
-		
+
 		/* perform residency calculations based on usecs */
 		FREQ = 1000;
 		return;
@@ -472,7 +539,7 @@ void start_timerstats(void)
 int line_compare (const void *av, const void *bv)
 {
 	const struct line	*a = av, *b = bv;
-	return b->count - a->count;
+	return (b->count + 50 * b->disk_count) - (a->count + 50 * a->disk_count);
 }
 
 void sort_lines(void)
@@ -526,14 +593,14 @@ int print_battery_proc_acpi(void)
 				continue;
 			c++;
 
-			if (strstr(line, "present voltage")) 
+			if (strstr(line, "present voltage"))
 				voltage = strtoull(c, NULL, 10) / 1000.0;
-		
+
 			if (strstr(line, "remaining capacity") && strstr(c, "mW"))
 				watts_left = strtoull(c, NULL, 10) / 1000.0;
 
 			if (strstr(line, "remaining capacity") && strstr(c, "mAh"))
-				amperes_left = strtoull(c, NULL, 10) / 1000.0; 
+				amperes_left = strtoull(c, NULL, 10) / 1000.0;
 
 			if (strstr(line, "present rate") && strstr(c, "mW"))
 				watts_drawn = strtoull(c, NULL, 10) / 1000.0 ;
@@ -543,12 +610,12 @@ int print_battery_proc_acpi(void)
 
 		}
 		fclose(file);
-	
+
 		if (!dontcount) {
 			rate += watts_drawn + voltage * amperes_drawn;
 		}
 		cap += watts_left + voltage * amperes_left;
-		
+
 
 	}
 	closedir(dir);
@@ -651,10 +718,10 @@ void print_battery_sysfs(void)
 	double cap = 0;
 
 	char filename[256];
-	
+
 	if (print_battery_proc_acpi())
 		return;
-	
+
 	if (print_battery_proc_pmu())
 		return;
 
@@ -712,14 +779,14 @@ void print_battery_sysfs(void)
 		if (!file) {
 			sprintf(filename, "/sys/class/power_supply/%s/charge_now", dirent->d_name);
 			file = fopen(filename, "r");
-			if (!file) 
+			if (!file)
 				continue;
 
 			/* W = A * V */
 			watts_left = voltage;
 		}
 		memset(line, 0, 1024);
-		if (fgets(line, 1024, file) != NULL) 
+		if (fgets(line, 1024, file) != NULL)
 			watts_left *= strtoull(line, NULL, 10) / 1000000.0;
 		fclose(file);
 
@@ -729,15 +796,15 @@ void print_battery_sysfs(void)
 			continue;
 		memset(line, 0, 1024);
 		if (fgets(line, 1024, file) != NULL) {
-			amperes_drawn = strtoull(line, NULL, 10) / 1000000.0;
+			watts_drawn = strtoull(line, NULL, 10) / 1000000.0;
 		}
 		fclose(file);
-	
+
 		if (!dontcount) {
 			rate += watts_drawn + voltage * amperes_drawn;
 		}
 		cap += watts_left;
-		
+
 
 	}
 	closedir(dir);
@@ -764,6 +831,7 @@ void usage()
 	printf(_("Usage: powertop [OPTION...]\n"));
 	printf(_("  -d, --dump            read wakeups once and print list of top offenders\n"));
 	printf(_("  -t, --time=DOUBLE     default time to gather data in seconds\n"));
+	printf(_("  -p, --pids            show pids in wakeups list\n"));
 	printf(_("  -h, --help            Show this help message\n"));
 	printf(_("  -v, --version         Show version information and exit\n"));
 	exit(0);
@@ -787,41 +855,47 @@ int main(int argc, char **argv)
 	bindtextdomain ("powertop", "/usr/share/locale");
 	textdomain ("powertop");
 
- 	while (1) {
- 		static struct option opts[] = {
- 			{ "dump", 0, NULL, 'd' },
- 			{ "time", 1, NULL, 't' },
- 			{ "help", 0, NULL, 'h' },
- 			{ "version", 0, NULL, 'v' },
- 			{ 0, 0, NULL, 0 }
- 		};
- 		int index2 = 0, c;
- 		
- 		c = getopt_long(argc, argv, "dt:hv", opts, &index2);
- 		if (c == -1)
- 			break;
- 		switch (c) {
- 		case 'd':
- 			dump = 1;
- 			break;
- 		case 't':
- 			ticktime = strtod(optarg, NULL);
- 			break;
- 		case 'h':
- 			usage();
- 			break;
- 		case 'v':
- 			version();
- 			break;
- 		default:
- 			;
- 		}
- 	}
+	start_data_dirty_capture();
+
+	while (1) {
+		static struct option opts[] = {
+			{ "dump", 0, NULL, 'd' },
+			{ "time", 1, NULL, 't' },
+			{ "pids", 0, NULL, 'p' },
+			{ "help", 0, NULL, 'h' },
+			{ "version", 0, NULL, 'v' },
+			{ 0, 0, NULL, 0 }
+		};
+		int index2 = 0, c;
+
+		c = getopt_long(argc, argv, "dt:phv", opts, &index2);
+		if (c == -1)
+			break;
+		switch (c) {
+		case 'd':
+			dump = 1;
+			break;
+		case 't':
+			ticktime = strtod(optarg, NULL);
+			break;
+		case 'p':
+			showpids = 1;
+			break;
+		case 'h':
+			usage();
+			break;
+		case 'v':
+			version();
+			break;
+		default:
+			;
+		}
+	}
 
 	if (!dump)
 		ticktime = 5.0;
- 
-	system("/sbin/modprobe cpufreq_stats &> /dev/null");
+
+	system("/sbin/modprobe cpufreq_stats > /dev/null 2>&1");
 	read_data(&start_usage[0], &start_duration[0]);
 
 
@@ -833,10 +907,17 @@ int main(int argc, char **argv)
 	do_cpufreq_stats();
 	count_usb_urbs();
 	count_usb_urbs();
+	count_device_pm();
+	count_device_pm();
+	do_alsa_stats();
+	do_alsa_stats();
+	do_ahci_stats();
+	do_ahci_stats();
+
 
 	memset(cur_usage, 0, sizeof(cur_usage));
 	memset(cur_duration, 0, sizeof(cur_duration));
-	printf("PowerTOP " VERSION "   (C) 2007, 2008 Intel Corporation \n\n");
+	printf("PowerTOP " VERSION "   (C) 2007 - 2010 Intel Corporation \n\n");
 	if (geteuid() != 0)
 		printf(_("PowerTOP needs to be run as root to collect enough information\n"));
 	printf(_("Collecting data for %i seconds \n"), (int)ticktime);
@@ -850,7 +931,7 @@ int main(int argc, char **argv)
 		int64_t totalevents;
 		fd_set rfds;
 		struct timeval tv;
-		int key;
+		int key = 0;
 
 		int i = 0;
 		double c0 = 0;
@@ -858,9 +939,10 @@ int main(int argc, char **argv)
 
 
 		FD_ZERO(&rfds);
-		FD_SET(0, &rfds);
+		if (!dump)
+			FD_SET(0, &rfds);
 		tv.tv_sec = ticktime;
-		tv.tv_usec = (ticktime - tv.tv_sec) * 1000000;;
+		tv.tv_usec = (ticktime - tv.tv_sec) * 1000000;
 		do_proc_irq();
 		start_timerstats();
 
@@ -885,7 +967,7 @@ int main(int argc, char **argv)
 
 		if (!dump) {
 			if (!ncursesinited) {
-				initialize_curses();  
+				initialize_curses();
 				ncursesinited++;
 			}
 			setup_windows();
@@ -897,7 +979,7 @@ int main(int argc, char **argv)
 		if (totalevents == 0 && maxcstate <= 1) {
 			sprintf(cstate_lines[5],_("< Detailed C-state information is not available.>\n"));
 		} else {
-			double sleept, percentage;;
+			double sleept, percentage;
 			c0 = sysconf(_SC_NPROCESSORS_ONLN) * ticktime * 1000 * FREQ - totalticks;
 			if (c0 < 0)
 				c0 = 0;	/* rounding errors in measurement might make c0 go slightly negative.. this is confusing */
@@ -924,7 +1006,7 @@ int main(int argc, char **argv)
 						maxsleep = sleept;
 					if (percentage > 50)
 						topcstate = i+1;
-					
+
 				}
 		}
 		do_cpufreq_stats();
@@ -938,9 +1020,10 @@ int main(int argc, char **argv)
 		while (file && !feof(file)) {
 			char *count, *pid, *process, *func;
 			char line2[1024];
-			int cnt;
+			int cnt = 0;
 			int deferrable = 0;
 			memset(line, 0, 1024);
+			memset(line2, 0, 1024);
 			if (fgets(line, 1024, file) == NULL)
 				break;
 			if (strstr(line, "total events"))
@@ -970,12 +1053,18 @@ int main(int argc, char **argv)
 			while (*c != 0 && *c == ' ')
 				c++;
 			func = c;
+
+			if (strcmp(process, "swapper")==0 &&
+			    strcmp(func, "hrtimer_start_range_ns (tick_sched_timer)\n")==0) {
+				process = _("[kernel scheduler]");
+				func = _("Load balancing tick");
+			}
 			if (strcmp(process, "insmod") == 0)
-				process = _("<kernel module>");
+				process = _("[kernel module]");
 			if (strcmp(process, "modprobe") == 0)
-				process = _("<kernel module>");
+				process = _("[kernel module]");
 			if (strcmp(process, "swapper") == 0)
-				process = _("<kernel core>");
+				process = _("[kernel core]");
 			c = strchr(c, '\n');
 			if (strncmp(func, "tick_nohz_", 10) == 0)
 				continue;
@@ -990,14 +1079,23 @@ int main(int argc, char **argv)
 				if (*c++ == 'D')
 					deferrable = 1;
 			}
+
 			if (deferrable)
 				continue;
-			sprintf(line2, "%15s : %s", process, func);
-			push_line_pid(line2, cnt, pid);
+			if (strchr(process, '['))
+				sprintf(line2, "%s %s", process, func);
+			else
+				sprintf(line2, "%s", process);
+			push_line_pid(line2, cnt, 0, pid);
 		}
+
 		if (file)
 			pclose(file);
 
+		reset_suggestions();
+
+		parse_data_dirty_buffer();
+
 		if (strstr(line, "total events")) {
 			int d;
 			d = strtoull(line, NULL, 10) / sysconf(_SC_NPROCESSORS_ONLN);
@@ -1007,15 +1105,19 @@ int main(int argc, char **argv)
 					totalevents += interrupt_0 - d;
 			}
 			if (d>0 && d < interrupt_0)
-				push_line(_("    <interrupt> : extra timer interrupt"), interrupt_0 - d);
+				push_line(_("[extra timer interrupt]"), interrupt_0 - d);
 		}
 
-	
+
 		if (totalevents && ticktime) {
 			wakeups_per_second = totalevents * 1.0 / ticktime / sysconf(_SC_NPROCESSORS_ONLN);
 			show_wakeups(wakeups_per_second, ticktime, c0 * 100.0 / (sysconf(_SC_NPROCESSORS_ONLN) * ticktime * 1000 * FREQ) );
 		}
 		count_usb_urbs();
+		count_device_pm();
+
+		do_alsa_stats();
+		do_ahci_stats();
 		print_battery_sysfs();
 		count_lines();
 		sort_lines();
@@ -1058,10 +1160,14 @@ int main(int argc, char **argv)
 		if (wakeups_per_second < 0)
 			ticktime = 2;
 
-		reset_suggestions();
+		reset_suggestions2();
 
 		suggest_kernel_config("CONFIG_USB_SUSPEND", 1,
 				    _("Suggestion: Enable the CONFIG_USB_SUSPEND kernel configuration option.\nThis option will automatically disable UHCI USB when not in use, and may\nsave approximately 1 Watt of power."), 20);
+		suggest_kernel_config("CONFIG_PM_ADVANCED_DEBUG", 1,
+				    _("Suggestion: Enable the CONFIG_PM_ADVANCED_DEBUG kernel configuration option.\nThis option will allow PowerTOP to collect runtime power management statistics."), 10);
+		suggest_kernel_config("CONFIG_PM_RUNTIME", 1,
+				    _("Suggestion: Enable the CONFIG_PM_RUNTIME kernel configuration option.\nThis option enables the kernel to manage power for various devices in your computer."), 40);
 		suggest_kernel_config("CONFIG_CPU_FREQ_GOV_ONDEMAND", 1,
 				    _("Suggestion: Enable the CONFIG_CPU_FREQ_GOV_ONDEMAND kernel configuration option.\n"
 				      "The 'ondemand' CPU speed governor will minimize the CPU power usage while\n" "giving you performance when it is needed."), 5);
@@ -1071,6 +1177,9 @@ int main(int argc, char **argv)
 		suggest_kernel_config("CONFIG_HPET_TIMER", 1,
 				    _("Suggestion: Enable the CONFIG_HPET_TIMER kernel configuration option.\n"
 				      "Without HPET support the kernel needs to wake up every 20 milliseconds for \n" "some housekeeping tasks."), 10);
+		suggest_kernel_config("CONFIG_PCIEASPM", 1,
+				    _("Suggestion: Enable the CONFIG_PCIEASPM kernel configuration option.\n"
+				      "PCI Link Powermanagement (ASPM) allows the hardware to go to low power mode\n" "automatically when a PCI-E device is idle."), 10);
 		if (!access("/sys/module/snd_ac97_codec", F_OK) &&
 			access("/sys/module/snd_ac97_codec/parameters/power_save", F_OK))
 			suggest_kernel_config("CONFIG_SND_AC97_POWER_SAVE", 1,
@@ -1084,7 +1193,7 @@ int main(int argc, char **argv)
 				      "This option allows PowerTOP to show P-state percentages \n" "P-states correspond to CPU frequencies."), 2);
 		suggest_kernel_config("CONFIG_INOTIFY", 1,
 				    _("Suggestion: Enable the CONFIG_INOTIFY kernel configuration option.\n"
-				      "This option allows programs to wait for changes in files and directories\n" 
+				      "This option allows programs to wait for changes in files and directories\n"
 				      "instead of having to poll for these changes"), 5);
 
 
@@ -1116,7 +1225,7 @@ int main(int argc, char **argv)
 
 		/* suggest to stop hal polilng if it shows up in the top 50 and wakes up too much*/
 		suggest_process_death("hald-addon-stor : ", "hald-addon-storage", lines, min(linehead,50), 2.0,
-				    _( "Suggestion: Disable 'hal' from polling your cdrom with:  \n" 
+				    _( "Suggestion: Disable 'hal' from polling your cdrom with:  \n"
 				       "hal-disable-polling --device /dev/cdrom 'hal' is the component that auto-opens a\n"
 				       "window if you plug in a CD but disables SATA power saving from kicking in."), 30);
 
@@ -1126,14 +1235,21 @@ int main(int argc, char **argv)
 				      "SE-Alert alerts you about SELinux policy violations, but also\n"
 				      "has a bug that wakes it up 10 times per second."), 20);
 
+		suggest_on_dmesg("failed to find CxSR latency, disabling CxSR",
+			_("The Intel Integrated Graphics driver failed to enable Memory "
+			  "self refresh.\nMemory Self Refresh is important for "
+			  "good memory power savings.\nPlease check your OS "
+			  "vendor for a kernel update and/or report a bug."),
+			  10);
 
 		suggest_bluetooth_off();
 		suggest_nmi_watchdog();
-		suggest_laptop_mode();
 		if (maxsleep > 15.0)
 			suggest_hpet();
 		suggest_ac97_powersave();
+		suggest_hda_powersave();
 		suggest_wireless_powersave();
+		suggest_wifi_new_powersave();
 		suggest_ondemand_governor();
 		suggest_noatime();
 		suggest_sata_alpm();
@@ -1142,11 +1258,19 @@ int main(int argc, char **argv)
 		suggest_WOL_off();
 		suggest_writeback_time();
 		suggest_usb_autosuspend();
+		suggest_runtime_suspend();
+
 		usb_activity_hint();
+		void devicepm_activity_hint();
+		alsa_activity_hint();
+		ahci_activity_hint();
 
 		if (dump) {
 			print_all_suggestions();
 			display_usb_activity();
+			display_runtime_activity();
+			display_alsa_activity();
+			display_ahci_activity();
 			exit(EXIT_SUCCESS);
 		}
 
@@ -1168,8 +1292,9 @@ int main(int argc, char **argv)
 		memcpy(last_duration, cur_duration, sizeof(last_duration));
 
 
-		
+
 	}
 
+	end_data_dirty_capture();
 	return 0;
 }
