@@ -19,7 +19,7 @@
  * Boston, MA 02110-1301 USA
  *
  * Authors:
- * 	Arjan van de Ven <arjan@linux.intel.com>
+ *	Arjan van de Ven <arjan@linux.intel.com>
  */
 
 
@@ -31,12 +31,13 @@
 struct line {
 	char	*string;
 	int	count;
-	char 	pid[12];
+	int	disk_count;
+	char	pid[12];
 };
 
 typedef void (suggestion_func)(void);
 
-extern struct line     *lines;  
+extern struct line     *lines;
 extern int             linehead;
 extern int             linesize;
 extern int             linectotal;
@@ -45,12 +46,13 @@ extern double displaytime;
 
 void suggest_process_death(char *process_match, char *process_name, struct line *slines, int linecount, double minwakeups, char *comment, int weight);
 void suggest_kernel_config(char *string, int onoff, char *comment, int weight);
-void suggest_laptop_mode(void);
 void suggest_bluetooth_off(void);
 void suggest_nmi_watchdog(void);
 void suggest_hpet(void);
 void suggest_ac97_powersave(void);
+void suggest_hda_powersave(void);
 void suggest_wireless_powersave(void);
+void suggest_wifi_new_powersave(void);
 void suggest_ondemand_governor(void);
 void suggest_noatime(void);
 void suggest_sata_alpm(void);
@@ -59,7 +61,11 @@ void suggest_xrandr_TV_off(void);
 void suggest_WOL_off(void);
 void suggest_writeback_time(void);
 void suggest_usb_autosuspend(void);
+void suggest_runtime_suspend(void);
+
 void usb_activity_hint(void);
+void devicepm_activity_hint(void);
+
 
 
 
@@ -68,14 +74,14 @@ extern char cstate_lines[12][200];
 extern char cpufreqstrings[6][80];
 
 extern int topcstate;
-extern int topfreq;  
+extern int topfreq;
 extern int dump;
 
 extern int showpids;
 
 extern char status_bar_slots[10][40];
 extern char suggestion_key;
-extern suggestion_func *suggestion_activate; 
+extern suggestion_func *suggestion_activate;
 
 
 /* min definition borrowed from the Linux kernel */
@@ -114,15 +120,41 @@ void show_suggestion(char *sug);
 void pick_suggestion(void);
 void add_suggestion(char *text, int weight, char key, char *keystring, suggestion_func *func);
 void reset_suggestions(void);
+void reset_suggestions2(void);
 void print_all_suggestions(void);
 void push_line(char *string, int count);
+void push_line_pid(char *string, int cpu_count, int disk_count, char *pid);
+
 
 void  do_cpufreq_stats(void);
 void count_usb_urbs(void);
+void count_device_pm(void);
+
+void alsa_activity_hint(void);
+void display_alsa_activity(void);
+void do_alsa_stats(void);
+
+void ahci_activity_hint(void);
+void display_ahci_activity(void);
+void do_ahci_stats(void);
+
+
 
 void display_usb_activity(void);
+void display_runtime_activity(void);
+
 void activate_usb_autosuspend(void);
 void print_intel_cstates(void);
 
+void start_data_dirty_capture(void);
+void end_data_dirty_capture(void);
+void parse_data_dirty_buffer(void);
+
+
+void hda_power_on(void);
+void activate_alpm(void);
+
+void suggest_on_dmesg(char *string, char *comment, int weight);
+
 
 #endif
